@@ -1,6 +1,7 @@
 import json
 import logging
 from abc import ABC, abstractmethod
+from pathlib import Path
 from typing import Any, Dict, Optional, cast
 
 import boto3
@@ -55,8 +56,8 @@ class FeatureToggle:
             return FeatureToggle.DIALUP_RESOLVER[dialup_type](
                 region_config, account_id=self.account_id, feature_name=feature_name
             )
-        LOG.warning("Dialup type '{}' is None or is not supported.".format(dialup_type))
-        return DisabledDialup(region_config)  # type: ignore[no-untyped-call]
+        LOG.warning(f"Dialup type '{dialup_type}' is None or is not supported.")
+        return DisabledDialup(region_config)
 
     def is_enabled(self, feature_name: str) -> bool:
         """
@@ -65,7 +66,7 @@ class FeatureToggle:
         :param feature_name: name of feature
         """
         if feature_name not in self.feature_config:
-            LOG.warning("Feature '{}' not available in Feature Toggle Config.".format(feature_name))
+            LOG.warning(f"Feature '{feature_name}' not available in Feature Toggle Config.")
             return False
 
         stage = self.stage
@@ -73,13 +74,13 @@ class FeatureToggle:
         account_id = self.account_id
         if not stage or not region or not account_id:
             LOG.warning(
-                "One or more of stage, region and account_id is not set. Feature '{}' not enabled.".format(feature_name)
+                f"One or more of stage, region and account_id is not set. Feature '{feature_name}' not enabled."
             )
             return False
 
         stage_config = self.feature_config.get(feature_name, {}).get(stage, {})
         if not stage_config:
-            LOG.info("Stage '{}' not enabled for Feature '{}'.".format(stage, feature_name))
+            LOG.info(f"Stage '{stage}' not enabled for Feature '{feature_name}'.")
             return False
 
         if account_id in stage_config:
@@ -89,10 +90,10 @@ class FeatureToggle:
             region_config = stage_config[region] if region in stage_config else stage_config.get("default", {})
 
         dialup = self._get_dialup(region_config, feature_name=feature_name)  # type: ignore[no-untyped-call]
-        LOG.info("Using Dialip {}".format(dialup))
+        LOG.info(f"Using Dialip {dialup}")
         is_enabled: bool = dialup.is_enabled()
 
-        LOG.info("Feature '{}' is enabled: '{}'".format(feature_name, is_enabled))
+        LOG.info(f"Feature '{feature_name}' is enabled: '{is_enabled}'")
         return is_enabled
 
 
@@ -119,10 +120,9 @@ class FeatureToggleDefaultConfigProvider(FeatureToggleConfigProvider):
 class FeatureToggleLocalConfigProvider(FeatureToggleConfigProvider):
     """Feature toggle config provider which uses a local file. This is to facilitate local testing."""
 
-    def __init__(self, local_config_path):  # type: ignore[no-untyped-def]
+    def __init__(self, local_config_path: str) -> None:
         FeatureToggleConfigProvider.__init__(self)
-        with open(local_config_path, "r", encoding="utf-8") as f:
-            config_json = f.read()
+        config_json = Path(local_config_path).read_text(encoding="utf-8")
         self.feature_toggle_config = cast(Dict[str, Any], json.loads(config_json))
 
     @property
@@ -134,7 +134,7 @@ class FeatureToggleAppConfigConfigProvider(FeatureToggleConfigProvider):
     """Feature toggle config provider which loads config from AppConfig."""
 
     @cw_timer(prefix="External", name="AppConfig")
-    def __init__(self, application_id, environment_id, configuration_profile_id, app_config_client=None):  # type: ignore[no-untyped-def]
+    def __init__(self, application_id, environment_id, configuration_profile_id, app_config_client=None) -> None:  # type: ignore[no-untyped-def]
         FeatureToggleConfigProvider.__init__(self)
         try:
             LOG.info("Loading feature toggle config from AppConfig...")
@@ -157,7 +157,7 @@ class FeatureToggleAppConfigConfigProvider(FeatureToggleConfigProvider):
             self.feature_toggle_config = cast(Dict[str, Any], json.loads(binary_config_string.decode("utf-8")))
             LOG.info("Finished loading feature toggle config from AppConfig.")
         except Exception as ex:
-            LOG.error("Failed to load config from AppConfig: {}. Using empty config.".format(ex))
+            LOG.error(f"Failed to load config from AppConfig: {ex}. Using empty config.")
             # There is chance that AppConfig is not available in a particular region.
             self.feature_toggle_config = {}
 
