@@ -1,11 +1,20 @@
 from collections import namedtuple
-from typing import Any, Dict, List, Optional
+from typing import Any, Dict, Iterable, List, Optional
+
+from typing_extensions import TypeGuard
 
 from samtranslator.model import ResourceResolver
-from samtranslator.model.intrinsics import get_logical_id_from_intrinsic, ref, fnGetAtt
+from samtranslator.model.apigateway import ApiGatewayRestApi
+from samtranslator.model.apigatewayv2 import ApiGatewayV2HttpApi
+from samtranslator.model.dynamodb import DynamoDBTable
+from samtranslator.model.intrinsics import fnGetAtt, get_logical_id_from_intrinsic, ref
+from samtranslator.model.lambda_ import (
+    LambdaFunction,
+)
+from samtranslator.model.stepfunctions import StepFunctionsStateMachine
+from samtranslator.public.sdk.resource import SamResourceType
 from samtranslator.utils.utils import as_array, insert_unique
 
-
 # TODO: Switch to dataclass
 ConnectorResourceReference = namedtuple(
     "ConnectorResourceReference",
@@ -21,6 +30,16 @@ ConnectorResourceReference = namedtuple(
     ],
 )
 
+_SAM_TO_CFN_RESOURCE_TYPE = {
+    SamResourceType.Function.value: LambdaFunction.resource_type,
+    SamResourceType.StateMachine.value: StepFunctionsStateMachine.resource_type,
+    SamResourceType.Api.value: ApiGatewayRestApi.resource_type,
+    SamResourceType.HttpApi.value: ApiGatewayV2HttpApi.resource_type,
+    SamResourceType.SimpleTable.value: DynamoDBTable.resource_type,
+}
+
+UNSUPPORTED_CONNECTOR_PROFILE_TYPE = "UNSUPPORTED_CONNECTOR_PROFILE_TYPE"
+
 
 class ConnectorResourceError(Exception):
     """
@@ -28,11 +47,11 @@ class ConnectorResourceError(Exception):
     """
 
 
-def _is_nonblank_str(s: Any) -> bool:
+def _is_nonblank_str(s: Any) -> TypeGuard[str]:
     return s and isinstance(s, str)
 
 
-def add_depends_on(logical_id: str, depends_on: str, resource_resolver: ResourceResolver):  # type: ignore[no-untyped-def]
+def add_depends_on(logical_id: str, depends_on: str, resource_resolver: ResourceResolver) -> None:
     """
     Add DependsOn attribute to resource.
     """
@@ -57,7 +76,9 @@ def replace_depends_on_logical_id(logical_id: str, replacement: List[str], resou
             resource["DependsOn"] = insert_unique(depends_on, replacement)
 
 
-def get_event_source_mappings(event_source_id: str, function_id: str, resource_resolver: ResourceResolver):  # type: ignore[no-untyped-def]
+def get_event_source_mappings(
+    event_source_id: str, function_id: str, resource_resolver: ResourceResolver
+) -> Iterable[str]:
     """
     Get logical IDs of `AWS::Lambda::EventSourceMapping`s between resource logical IDs.
     """
@@ -80,8 +101,9 @@ def get_event_source_mappings(event_source_id: str, function_id: str, resource_r
 
 def _is_valid_resource_reference(obj: Dict[str, Any]) -> bool:
     id_provided = "Id" in obj
-    non_id_provided = len([k for k in obj.keys() if k != "Id"]) > 0
-    # Must provide either Id, or a combination of other properties, but not both
+    # Every property in ResourceReference can be implied using 'Id', except for 'Qualifier', so users should be able to combine 'Id' and 'Qualifier'
+    non_id_provided = len([k for k in obj if k not in ["Id", "Qualifier"]]) > 0
+    # Must provide Id (with optional Qualifier) or a supported combination of other properties.
     return id_provided != non_id_provided
 
 
@@ -89,17 +111,21 @@ def get_resource_reference(
     obj: Dict[str, Any], resource_resolver: ResourceResolver, connecting_obj: Dict[str, Any]
 ) -> ConnectorResourceReference:
     if not _is_valid_resource_reference(obj):
-        raise ConnectorResourceError("Must provide either 'Id' or a combination of the other properties, not both.")
+        raise ConnectorResourceError(
+            "Must provide 'Id' (with optional 'Qualifier') or a supported combination of other properties."
+        )
 
     logical_id = obj.get("Id")
-
-    # Must either provide Id or a combination of the other properties (not both).
+    # Must provide Id (with optional Qualifier) or a supported combination of other properties
     # If Id is not provided, all values must come from overrides.
     if not logical_id:
         resource_type = obj.get("Type")
         if not _is_nonblank_str(resource_type):
             raise ConnectorResourceError("'Type' is missing or not a string.")
-        resource_type = str(resource_type)
+
+        # profiles.json only support CFN resource type.
+        # We need to convert SAM resource types to corresponding CFN resource type
+        resource_type = _SAM_TO_CFN_RESOURCE_TYPE.get(resource_type, resource_type)
 
         return ConnectorResourceReference(
             logical_id=None,
@@ -122,8 +148,6 @@ def get_resource_reference(
     resource_type = resource.get("Type")
     if not _is_nonblank_str(resource_type):
         raise ConnectorResourceError("'Type' is missing or not a string.")
-    resource_type = str(resource_type)
-
     properties = resource.get("Properties", {})
 
     arn = _get_resource_arn(logical_id, resource_type)
@@ -136,7 +160,7 @@ def get_resource_reference(
 
     name = _get_resource_name(logical_id, resource_type)
 
-    qualifier = _get_resource_qualifier(resource_type)
+    qualifier = obj.get("Qualifier") if "Qualifier" in obj else _get_resource_qualifier(resource_type)
 
     return ConnectorResourceReference(
         logical_id=logical_id,
@@ -157,6 +181,8 @@ def _get_resource_role_property(
         return properties.get("Role")
     if resource_type == "AWS::StepFunctions::StateMachine":
         return properties.get("RoleArn")
+    if resource_type == "AWS::AppSync::DataSource":
+        return properties.get("ServiceRoleArn")
     if resource_type == "AWS::Events::Rule":
         for target in properties.get("Targets", []):
             target_arn = target.get("Arn")
@@ -182,25 +208,29 @@ def _get_resource_role_name(
     return ref(logical_id)
 
 
-def _get_resource_queue_url(logical_id: str, resource_type: str) -> Any:
+def _get_resource_queue_url(logical_id: str, resource_type: str) -> Optional[Dict[str, Any]]:
     if resource_type == "AWS::SQS::Queue":
         return ref(logical_id)
+    return None
 
 
-def _get_resource_id(logical_id: str, resource_type: str) -> Any:
+def _get_resource_id(logical_id: str, resource_type: str) -> Optional[Dict[str, Any]]:
     if resource_type in ["AWS::ApiGateway::RestApi", "AWS::ApiGatewayV2::Api"]:
         return ref(logical_id)
+    return None
 
 
-def _get_resource_name(logical_id: str, resource_type: str) -> Any:
+def _get_resource_name(logical_id: str, resource_type: str) -> Optional[Dict[str, Any]]:
     if resource_type == "AWS::StepFunctions::StateMachine":
         return fnGetAtt(logical_id, "Name")
+    return None
 
 
-def _get_resource_qualifier(resource_type: str) -> Any:
+def _get_resource_qualifier(resource_type: str) -> Optional[str]:
     # Qualifier is used as the execute-api ARN suffix; by default allow whole API
     if resource_type in ["AWS::ApiGateway::RestApi", "AWS::ApiGatewayV2::Api"]:
         return "*"
+    return None
 
 
 def _get_resource_arn(logical_id: str, resource_type: str) -> Any:
