@@ -1,16 +1,17 @@
 import copy
 import json
-import os
 import re
+from pathlib import Path
 from typing import Any, Dict
 
 ConnectorProfile = Dict[str, Any]
 
-with open(os.path.join(os.path.dirname(os.path.abspath(__file__)), "profiles.json")) as f:
+_PROFILE_FILE = Path(__file__).absolute().parent / "profiles.json"
+with _PROFILE_FILE.open(encoding="utf-8") as f:
     PROFILE: ConnectorProfile = json.load(f)
 
 
-def get_profile(source_type: str, dest_type: str):
+def get_profile(source_type: str, dest_type: str):  # type: ignore[no-untyped-def]
     profile = PROFILE["Permissions"].get(source_type, {}).get(dest_type)
     # Ensure not passing a mutable shared variable
     return copy.deepcopy(profile)
@@ -26,7 +27,7 @@ def verify_profile_variables_replaced(obj: Any) -> None:
         raise ValueError(f"The following variables have not been replaced: {matches}")
 
 
-def profile_replace(obj: Any, replacements: Dict[str, Any]):
+def profile_replace(obj: Any, replacements: Dict[str, Any]):  # type: ignore[no-untyped-def]
     """
     This function is used to recursively replace all keys in 'replacements' found
     in 'obj' with matching values in 'replacement' dictionary.
@@ -37,13 +38,12 @@ def profile_replace(obj: Any, replacements: Dict[str, Any]):
     return _map_nested(obj, lambda v: _profile_replace_str(v, replacements))
 
 
-def _map_nested(obj: Any, fn):
+def _map_nested(obj: Any, fn):  # type: ignore[no-untyped-def, no-untyped-def]
     if isinstance(obj, dict):
         return {k: _map_nested(v, fn) for k, v in obj.items()}
-    elif isinstance(obj, list):
+    if isinstance(obj, list):
         return [_map_nested(v, fn) for v in obj]
-    else:
-        return fn(obj)
+    return fn(obj)
 
 
 def _sanitize(s: str) -> str:
@@ -51,7 +51,7 @@ def _sanitize(s: str) -> str:
     return "".join(c for c in s if c.isalnum())
 
 
-def _profile_replace_str(s: Any, replacements: Dict[str, Any]):
+def _profile_replace_str(s: Any, replacements: Dict[str, Any]):  # type: ignore[no-untyped-def]
     if not isinstance(s, str):
         return s
     res = {}
