@@ -1,4 +1,4 @@
-﻿""" SAM macro definitions """
+""" SAM macro definitions """
 import copy
 from contextlib import suppress
 from typing import Any, Callable, Dict, List, Optional, Tuple, Union, cast
@@ -75,7 +75,18 @@ from samtranslator.model.role_utils import construct_role_for_resource
 from samtranslator.model.sns import SNSTopic, SNSTopicPolicy
 from samtranslator.model.sqs import SQSQueue, SQSQueuePolicy
 from samtranslator.model.stepfunctions import StateMachineGenerator
-from samtranslator.model.types import IS_DICT, IS_STR, PassThrough, any_type, dict_of, is_type, list_of, one_of
+from samtranslator.model.types import (
+    IS_BOOL,
+    IS_DICT,
+    IS_INT,
+    IS_LIST,
+    IS_STR,
+    PassThrough,
+    any_type,
+    dict_of,
+    list_of,
+    one_of,
+)
 from samtranslator.model.xray_utils import get_xray_managed_policy_name
 from samtranslator.translator import logical_id_generator
 from samtranslator.translator.arn_generator import ArnGenerator
@@ -126,7 +137,7 @@ class SamFunction(SamResourceMacro):
         # Intrinsic functions in value of Alias property are not supported, yet
         "AutoPublishAlias": PropertyType(False, one_of(IS_STR)),
         "AutoPublishCodeSha256": PropertyType(False, one_of(IS_STR)),
-        "AutoPublishAliasAllProperties": Property(False, is_type(bool)),
+        "AutoPublishAliasAllProperties": Property(False, IS_BOOL),
         "VersionDescription": PassThroughProperty(False),
         "ProvisionedConcurrencyConfig": PassThroughProperty(False),
         "FileSystemConfigs": PassThroughProperty(False),
@@ -321,7 +332,7 @@ class SamFunction(SamResourceMacro):
         # Try to resolve.
         resolved_event_invoke_config = intrinsics_resolver.resolve_parameter_refs(self.EventInvokeConfig)
 
-        logical_id = "{id}EventInvokeConfig".format(id=function_name)
+        logical_id = f"{function_name}EventInvokeConfig"
         lambda_event_invoke_config = (
             LambdaEventInvokeConfig(
                 logical_id=logical_id, depends_on=[lambda_alias.logical_id], attributes=self.resource_attributes
@@ -413,7 +424,7 @@ class SamFunction(SamResourceMacro):
                 policy = self._add_event_invoke_managed_policy(dest_config, resource_logical_id, destination)
             else:
                 raise InvalidResourceException(
-                    self.logical_id, "Destination is required if Type is not {}".format(auto_inject_list)
+                    self.logical_id, f"Destination is required if Type is not {auto_inject_list}"
                 )
         if dest_config.get("Destination") is not None and property_condition is None:
             policy = self._add_event_invoke_managed_policy(
@@ -496,7 +507,7 @@ class SamFunction(SamResourceMacro):
         if not isinstance(resolved_alias_name, str):
             # This is still a dictionary which means we are not able to completely resolve intrinsics
             raise InvalidResourceException(
-                self.logical_id, "'{}' must be a string or a Ref to a template parameter".format(property_name)
+                self.logical_id, f"'{property_name}' must be a string or a Ref to a template parameter"
             )
 
         return resolved_alias_name
@@ -631,20 +642,20 @@ class SamFunction(SamResourceMacro):
         if packagetype not in [ZIP, IMAGE]:
             raise InvalidResourceException(
                 lambda_function.logical_id,
-                "PackageType needs to be `{zip}` or `{image}`".format(zip=ZIP, image=IMAGE),
+                f"PackageType needs to be `{ZIP}` or `{IMAGE}`",
             )
 
         def _validate_package_type_zip() -> None:
             if not all([lambda_function.Runtime, lambda_function.Handler]):
                 raise InvalidResourceException(
                     lambda_function.logical_id,
-                    "Runtime and Handler needs to be present when PackageType is of type `{zip}`".format(zip=ZIP),
+                    f"Runtime and Handler needs to be present when PackageType is of type `{ZIP}`",
                 )
 
             if any([lambda_function.Code.get("ImageUri", False), lambda_function.ImageConfig]):
                 raise InvalidResourceException(
                     lambda_function.logical_id,
-                    "ImageUri or ImageConfig cannot be present when PackageType is of type `{zip}`".format(zip=ZIP),
+                    f"ImageUri or ImageConfig cannot be present when PackageType is of type `{ZIP}`",
                 )
 
         def _validate_package_type_image() -> None:
@@ -658,7 +669,7 @@ class SamFunction(SamResourceMacro):
             if not lambda_function.Code.get("ImageUri"):
                 raise InvalidResourceException(
                     lambda_function.logical_id,
-                    "ImageUri needs to be present when PackageType is of type `{image}`".format(image=IMAGE),
+                    f"ImageUri needs to be present when PackageType is of type `{IMAGE}`",
                 )
 
         _validate_per_package_type = {ZIP: _validate_package_type_zip, IMAGE: _validate_package_type_image}
@@ -683,9 +694,7 @@ class SamFunction(SamResourceMacro):
 
         # Validate required Types
         if dlq_type not in self.dead_letter_queue_policy_actions:
-            raise InvalidResourceException(
-                self.logical_id, "'DeadLetterQueue' requires Type of {}".format(valid_dlq_types)
-            )
+            raise InvalidResourceException(self.logical_id, f"'DeadLetterQueue' requires Type of {valid_dlq_types}")
 
     def _event_resources_to_link(self, resources: Dict[str, Any]) -> Dict[str, Any]:
         event_resources = {}
@@ -696,7 +705,7 @@ class SamFunction(SamResourceMacro):
                         self.logical_id + logical_id, event_dict, logical_id
                     )
                 except (TypeError, AttributeError) as e:
-                    raise InvalidEventException(logical_id, "{}".format(e)) from e
+                    raise InvalidEventException(logical_id, f"{e}") from e
                 event_resources[logical_id] = event_source.resources_to_link(resources)
         return event_resources
 
@@ -744,7 +753,7 @@ class SamFunction(SamResourceMacro):
                         lambda_function.logical_id + logical_id, event_dict, logical_id
                     )
                 except TypeError as e:
-                    raise InvalidEventException(logical_id, "{}".format(e)) from e
+                    raise InvalidEventException(logical_id, f"{e}") from e
 
                 kwargs = {
                     # When Alias is provided, connect all event sources to the alias and *not* the function
@@ -776,7 +785,7 @@ class SamFunction(SamResourceMacro):
             artifacts = {"ImageUri": self.ImageUri}
 
         if packagetype not in [ZIP, IMAGE]:
-            raise InvalidResourceException(self.logical_id, "invalid 'PackageType' : {}".format(packagetype))
+            raise InvalidResourceException(self.logical_id, f"invalid 'PackageType' : {packagetype}")
 
         # Inline function for transformation of inline code.
         # It accepts arbitrary argumemnts, because the arguments do not matter for the result.
@@ -855,7 +864,7 @@ class SamFunction(SamResourceMacro):
         #
         # SHA Collisions: For purposes of triggering a new update, we are concerned about just the difference previous
         #                 and next hashes. The chances that two subsequent hashes collide is fairly low.
-        prefix = "{id}Version".format(id=self.logical_id)
+        prefix = f"{self.logical_id}Version"
         logical_dict = {}
         # We can't directly change AutoPublishAlias as that would be a breaking change, so we have to add this opt-in
         # property that when set to true would change the lambda version whenever a property in the lambda function changes
@@ -899,7 +908,7 @@ class SamFunction(SamResourceMacro):
         if not name:
             raise InvalidResourceException(self.logical_id, "Alias name is required to create an alias")
 
-        logical_id = "{id}Alias{suffix}".format(id=function.logical_id, suffix=name)
+        logical_id = f"{function.logical_id}Alias{name}"
         alias = LambdaAlias(logical_id=logical_id, attributes=self.get_passthrough_resource_attributes())
         alias.Name = name
         alias.FunctionName = function.get_runtime_attr("name")
@@ -1034,6 +1043,7 @@ class SamFunction(SamResourceMacro):
         lambda_url.TargetFunctionArn = (
             lambda_alias.get_runtime_attr("arn") if lambda_alias else lambda_function.get_runtime_attr("name")
         )
+        lambda_url.InvokeMode = function_url_config.get("InvokeMode")
         return lambda_url
 
     def _validate_function_url_params(
@@ -1085,7 +1095,7 @@ class SamFunction(SamResourceMacro):
             if prop_name not in cors_property_data_type:
                 raise InvalidResourceException(
                     lambda_function.logical_id,
-                    "{} is not a valid property for configuring Cors.".format(prop_name),
+                    f"{prop_name} is not a valid property for configuring Cors.",
                 )
             prop_type = cors_property_data_type.get(prop_name, list)
             if not is_intrinsic(prop_value) and not isinstance(prop_value, prop_type):
@@ -1141,34 +1151,35 @@ class SamApi(SamResourceMacro):
         # Implicit APIs. For Explicit APIs, customer is expected to set integration URI themselves.
         # In the future, we might rename and expose this property to customers so they can have SAM manage Explicit APIs
         # Swagger.
-        "__MANAGE_SWAGGER": PropertyType(False, is_type(bool)),
+        "__MANAGE_SWAGGER": PropertyType(False, IS_BOOL),
         "Name": PropertyType(False, one_of(IS_STR, IS_DICT)),
         "StageName": PropertyType(True, one_of(IS_STR, IS_DICT)),
         "Tags": PropertyType(False, IS_DICT),
         "DefinitionBody": PropertyType(False, IS_DICT),
         "DefinitionUri": PropertyType(False, one_of(IS_STR, IS_DICT)),
-        "MergeDefinitions": Property(False, is_type(bool)),
-        "CacheClusterEnabled": PropertyType(False, is_type(bool)),
+        "MergeDefinitions": Property(False, IS_BOOL),
+        "CacheClusterEnabled": PropertyType(False, IS_BOOL),
         "CacheClusterSize": PropertyType(False, IS_STR),
         "Variables": PropertyType(False, IS_DICT),
         "EndpointConfiguration": PropertyType(False, one_of(IS_STR, IS_DICT)),
-        "MethodSettings": PropertyType(False, is_type(list)),
-        "BinaryMediaTypes": PropertyType(False, is_type(list)),
-        "MinimumCompressionSize": PropertyType(False, is_type(int)),
+        "MethodSettings": PropertyType(False, IS_LIST),
+        "BinaryMediaTypes": PropertyType(False, IS_LIST),
+        "MinimumCompressionSize": PropertyType(False, IS_INT),
         "Cors": PropertyType(False, one_of(IS_STR, IS_DICT)),
         "Auth": PropertyType(False, IS_DICT),
         "GatewayResponses": PropertyType(False, IS_DICT),
         "AccessLogSetting": PropertyType(False, IS_DICT),
         "CanarySetting": PropertyType(False, IS_DICT),
-        "TracingEnabled": PropertyType(False, is_type(bool)),
+        "TracingEnabled": PropertyType(False, IS_BOOL),
         "OpenApiVersion": PropertyType(False, IS_STR),
         "Models": PropertyType(False, IS_DICT),
         "Domain": PropertyType(False, IS_DICT),
-        "FailOnWarnings": PropertyType(False, is_type(bool)),
+        "FailOnWarnings": PropertyType(False, IS_BOOL),
         "Description": PropertyType(False, IS_STR),
         "Mode": PropertyType(False, IS_STR),
-        "DisableExecuteApiEndpoint": PropertyType(False, is_type(bool)),
+        "DisableExecuteApiEndpoint": PropertyType(False, IS_BOOL),
         "ApiKeySourceType": PropertyType(False, IS_STR),
+        "AlwaysDeploy": Property(False, IS_BOOL),
     }
 
     Name: Optional[Intrinsicable[str]]
@@ -1198,6 +1209,7 @@ class SamApi(SamResourceMacro):
     Mode: Optional[Intrinsicable[str]]
     DisableExecuteApiEndpoint: Optional[Intrinsicable[bool]]
     ApiKeySourceType: Optional[Intrinsicable[str]]
+    AlwaysDeploy: Optional[bool]
 
     referable_properties = {
         "Stage": ApiGatewayStage.resource_type,
@@ -1209,7 +1221,7 @@ class SamApi(SamResourceMacro):
     }
 
     @cw_timer
-    def to_cloudformation(self, **kwargs):  # type: ignore[no-untyped-def]
+    def to_cloudformation(self, **kwargs) -> List[Resource]:  # type: ignore[no-untyped-def]
         """Returns the API Gateway RestApi, Deployment, and Stage to which this SAM Api corresponds.
 
         :param dict kwargs: already-converted resources that may need to be modified when converting this \
@@ -1217,7 +1229,6 @@ class SamApi(SamResourceMacro):
         :returns: a list of vanilla CloudFormation Resources, to which this Function expands
         :rtype: list
         """
-        resources = []
 
         intrinsics_resolver = kwargs["intrinsics_resolver"]
         self.BinaryMediaTypes = intrinsics_resolver.resolve_parameter_refs(self.BinaryMediaTypes)
@@ -1262,31 +1273,10 @@ class SamApi(SamResourceMacro):
             description=self.Description,
             mode=self.Mode,
             api_key_source_type=self.ApiKeySourceType,
+            always_deploy=self.AlwaysDeploy,
         )
 
-        (
-            rest_api,
-            deployment,
-            stage,
-            permissions,
-            domain,
-            basepath_mapping,
-            route53,
-            usage_plan_resources,
-        ) = api_generator.to_cloudformation(redeploy_restapi_parameters, route53_record_set_groups)
-
-        resources.extend([rest_api, deployment, stage])
-        resources.extend(permissions)
-        if domain:
-            resources.extend([domain])
-        if basepath_mapping:
-            resources.extend(basepath_mapping)
-        if route53:
-            resources.extend([route53])
-        # contains usage plan, api key and usageplan key resources
-        if usage_plan_resources:
-            resources.extend(usage_plan_resources)
-        return resources
+        return api_generator.to_cloudformation(redeploy_restapi_parameters, route53_record_set_groups)
 
 
 class SamHttpApi(SamResourceMacro):
@@ -1299,22 +1289,22 @@ class SamHttpApi(SamResourceMacro):
         # Implicit APIs. For Explicit APIs, this is managed by the DefaultDefinitionBody Plugin.
         # In the future, we might rename and expose this property to customers so they can have SAM manage Explicit APIs
         # Swagger.
-        "__MANAGE_SWAGGER": PropertyType(False, is_type(bool)),
+        "__MANAGE_SWAGGER": PropertyType(False, IS_BOOL),
         "Name": PassThroughProperty(False),
         "StageName": PropertyType(False, one_of(IS_STR, IS_DICT)),
         "Tags": PropertyType(False, IS_DICT),
         "DefinitionBody": PropertyType(False, IS_DICT),
         "DefinitionUri": PropertyType(False, one_of(IS_STR, IS_DICT)),
         "StageVariables": PropertyType(False, IS_DICT),
-        "CorsConfiguration": PropertyType(False, one_of(is_type(bool), IS_DICT)),
+        "CorsConfiguration": PropertyType(False, one_of(IS_BOOL, IS_DICT)),
         "AccessLogSettings": PropertyType(False, IS_DICT),
         "DefaultRouteSettings": PropertyType(False, IS_DICT),
         "Auth": PropertyType(False, IS_DICT),
         "RouteSettings": PropertyType(False, IS_DICT),
         "Domain": PropertyType(False, IS_DICT),
-        "FailOnWarnings": PropertyType(False, is_type(bool)),
+        "FailOnWarnings": PropertyType(False, IS_BOOL),
         "Description": PropertyType(False, IS_STR),
-        "DisableExecuteApiEndpoint": PropertyType(False, is_type(bool)),
+        "DisableExecuteApiEndpoint": PropertyType(False, IS_BOOL),
     }
 
     Name: Optional[Any]
@@ -1380,9 +1370,12 @@ class SamHttpApi(SamResourceMacro):
             domain,
             basepath_mapping,
             route53,
+            permissions,
         ) = api_generator.to_cloudformation(kwargs.get("route53_record_set_groups", {}))
 
         resources.append(http_api)
+        if permissions:
+            resources.extend(permissions)
         if domain:
             resources.append(domain)
         if basepath_mapping:
@@ -1403,7 +1396,7 @@ class SamSimpleTable(SamResourceMacro):
     resource_type = "AWS::Serverless::SimpleTable"
     property_types = {
         "PrimaryKey": PropertyType(False, dict_of(IS_STR, IS_STR)),
-        "ProvisionedThroughput": PropertyType(False, dict_of(IS_STR, one_of(is_type(int), IS_DICT))),
+        "ProvisionedThroughput": PropertyType(False, dict_of(IS_STR, one_of(IS_INT, IS_DICT))),
         "TableName": PropertyType(False, one_of(IS_STR, IS_DICT)),
         "Tags": PropertyType(False, IS_DICT),
         "SSESpecification": PropertyType(False, IS_DICT),
@@ -1463,7 +1456,7 @@ class SamSimpleTable(SamResourceMacro):
     def _convert_attribute_type(self, attribute_type: str) -> str:
         if attribute_type in self.attribute_type_conversions:
             return self.attribute_type_conversions[attribute_type]
-        raise InvalidResourceException(self.logical_id, "Invalid 'Type' \"{actual}\".".format(actual=attribute_type))
+        raise InvalidResourceException(self.logical_id, f"Invalid 'Type' \"{attribute_type}\".")
 
 
 class SamApplication(SamResourceMacro):
@@ -1481,7 +1474,7 @@ class SamApplication(SamResourceMacro):
         "Parameters": PropertyType(False, IS_DICT),
         "NotificationARNs": PropertyType(False, list_of(one_of(IS_STR, IS_DICT))),
         "Tags": PropertyType(False, IS_DICT),
-        "TimeoutInMinutes": PropertyType(False, is_type(int)),
+        "TimeoutInMinutes": PropertyType(False, IS_INT),
     }
 
     Location: Union[str, Dict[str, Any]]
@@ -1664,7 +1657,7 @@ class SamLayerVersion(SamResourceMacro):
                 return option
         raise InvalidResourceException(
             self.logical_id,
-            "'RetentionPolicy' must be one of the following options: {}.".format([self.RETAIN, self.DELETE]),
+            f"'RetentionPolicy' must be one of the following options: {[self.RETAIN, self.DELETE]}.",
         )
 
     def _validate_architectures(self, lambda_layer: LambdaLayerVersion) -> None:
@@ -1689,7 +1682,7 @@ class SamLayerVersion(SamResourceMacro):
             if not is_intrinsic(arq) and arq not in [ARM64, X86_64]:
                 raise InvalidResourceException(
                     lambda_layer.logical_id,
-                    "CompatibleArchitectures needs to be a list of '{}' or '{}'".format(X86_64, ARM64),
+                    f"CompatibleArchitectures needs to be a list of '{X86_64}' or '{ARM64}'",
                 )
 
 
@@ -1781,7 +1774,7 @@ class SamStateMachine(SamResourceMacro):
                         self.logical_id + logical_id, event_dict, logical_id
                     )
                 except (TypeError, AttributeError) as e:
-                    raise InvalidEventException(logical_id, "{}".format(e)) from e
+                    raise InvalidEventException(logical_id, f"{e}") from e
                 event_resources[logical_id] = event_source.resources_to_link(resources)
         return event_resources
 
