@@ -5,17 +5,18 @@ from typing import Any, Dict, Optional, cast
 from samtranslator.metrics.method_decorator import cw_timer
 from samtranslator.model import Property, PropertyType, Resource, ResourceMacro
 from samtranslator.model.eventbridge_utils import EventBridgeRuleUtils
-from samtranslator.model.events import EventsRule
+from samtranslator.model.events import EventsRule, generate_valid_target_id
 from samtranslator.model.eventsources.push import Api as PushApi
 from samtranslator.model.exceptions import InvalidEventException
 from samtranslator.model.iam import IAMRole, IAMRolePolicies
 from samtranslator.model.intrinsics import fnSub
-from samtranslator.model.types import IS_DICT, IS_STR, PassThrough, is_type
+from samtranslator.model.types import IS_BOOL, IS_DICT, IS_STR, PassThrough
 from samtranslator.swagger.swagger import SwaggerEditor
 from samtranslator.translator import logical_id_generator
 
 CONDITION = "Condition"
 SFN_EVETSOURCE_METRIC_PREFIX = "SFNEventSource"
+EVENT_RULE_SFN_TARGET_SUFFIX = "StepFunctionsTarget"
 
 
 class EventSource(ResourceMacro, metaclass=ABCMeta):
@@ -63,7 +64,7 @@ class EventSource(ResourceMacro, metaclass=ABCMeta):
         """
         role_logical_id = self._generate_logical_id(prefix=prefix, suffix=suffix, resource_type="Role")  # type: ignore[no-untyped-call]
         event_role = IAMRole(role_logical_id, attributes=resource.get_passthrough_resource_attributes())
-        event_role.AssumeRolePolicyDocument = IAMRolePolicies.construct_assume_role_policy_for_service_principal(  # type: ignore[no-untyped-call]
+        event_role.AssumeRolePolicyDocument = IAMRolePolicies.construct_assume_role_policy_for_service_principal(
             self.principal
         )
         state_machine_arn = resource.get_runtime_attr("arn")
@@ -85,7 +86,7 @@ class Schedule(EventSource):
     property_types = {
         "Schedule": PropertyType(True, IS_STR),
         "Input": PropertyType(False, IS_STR),
-        "Enabled": PropertyType(False, is_type(bool)),
+        "Enabled": PropertyType(False, IS_BOOL),
         "State": PropertyType(False, IS_STR),
         "Name": PropertyType(False, IS_STR),
         "Description": PropertyType(False, IS_STR),
@@ -156,7 +157,9 @@ class Schedule(EventSource):
         :rtype: dict
         """
         target_id = (
-            self.Target["Id"] if self.Target and "Id" in self.Target else self.logical_id + "StepFunctionsTarget"
+            self.Target["Id"]
+            if self.Target and "Id" in self.Target
+            else generate_valid_target_id(self.logical_id, EVENT_RULE_SFN_TARGET_SUFFIX)
         )
         target = {
             "Arn": resource.get_runtime_attr("arn"),
@@ -249,7 +252,9 @@ class CloudWatchEvent(EventSource):
         :rtype: dict
         """
         target_id = (
-            self.Target["Id"] if self.Target and "Id" in self.Target else self.logical_id + "StepFunctionsTarget"
+            self.Target["Id"]
+            if self.Target and "Id" in self.Target
+            else generate_valid_target_id(self.logical_id, EVENT_RULE_SFN_TARGET_SUFFIX)
         )
         target = {
             "Arn": resource.get_runtime_attr("arn"),
@@ -289,7 +294,7 @@ class Api(EventSource):
         "RestApiId": PropertyType(True, IS_STR),
         "Stage": PropertyType(False, IS_STR),
         "Auth": PropertyType(False, IS_DICT),
-        "UnescapeMappingTemplate": Property(False, is_type(bool)),
+        "UnescapeMappingTemplate": Property(False, IS_BOOL),
     }
 
     Path: str
